@@ -19,6 +19,8 @@ def getslaveinfoline(node):
 
 
 class BaseReport(object):
+    when = None
+
     def __init__(self, **kw):
         self.__dict__.update(kw)
 
@@ -158,17 +160,9 @@ class TestReport(BaseReport):
         )
 
 
-class TeardownErrorReport(BaseReport):
-    outcome = "failed"
-    when = "teardown"
-
-    def __init__(self, longrepr, **extra):
-        self.longrepr = longrepr
-        self.sections = []
-        self.__dict__.update(extra)
-
-
 class CollectReport(BaseReport):
+    when = "collect"
+
     def __init__(self, nodeid, outcome, longrepr, result, sections=(), **extra):
         self.nodeid = nodeid
         self.outcome = outcome
